@@ -511,18 +511,18 @@ def load_conflict(data) :
                     conflicts.append([g, id, d[idx1:idx2]])
     return conflicts
 
-def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities, scores, encodes) :
-    ortho_groups = np.vstack([ortho_groups[:, :2], ortho_groups[:, [1,0]]])
-    conflicts = {}
-    new_groups = {}
+def filt_genes(groups, ortho_groups, global_file, cfl_file, priorities, scores, encodes) :
+    ortho_groups = np.vstack([ortho_groups[:, :2], ortho_groups[:, [1,0]]])*1000
+    priorities = { k*1000:v for k, v in priorities.items() }
+    scores = { k*1000:v for k, v in scores.items() }
+    conflicts, new_groups = {}, {}
+
     encodes = np.array([n for i, n in sorted([[i, n] for n, i in encodes.items()])])
-    
-    clust_ref = { int(n):s for n, s in readFasta(params['clust']).items()}
-    
+    clust_ref = { int(n)*1000:s for n, s in readFasta(params['clust']).items()}
     used, pangenome, panList = {}, {}, {}
     lowest_p = max([v[0] for v in priorities.values()])    
     while len(scores) > 0 :
-        # get top 200 genes
+        # get top 500 genes
         ortho_groups = ortho_groups[np.all(in1d(ortho_groups, list(scores.keys())).reshape(ortho_groups.shape), 1)]
         genes = get_gene(scores, priorities, ortho_groups, cnt=500)
         if len(genes) <= 0 :
@@ -535,7 +535,7 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
         for gene, score in list(genes.items()) :
             present_iden = 0
             if gene not in new_groups :
-                mat = groups.get(gene)
+                mat = groups.get(int(gene/1000))
                 mat.T[4] = (10000 * mat.T[3]/np.max(mat.T[3])).astype(int)
                 for m in mat :
                     v = used.get(m[5], None)
@@ -563,14 +563,14 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
                        int(i) < int(i + chunk_size)]
             for cfl in pool2.imap_unordered(load_conflict, [ [cfl_file, ids] for ids in tab_ids ]) :
                 for g, i, c in cfl :
-                    conflicts[g][i] = c
+                    conflicts[g*1000][i] = c
         
         used2 = set([])
         for gene, score in sorted(genes.items(), key=lambda x:-x[1]) :
             if gene not in new_groups :
                 mat = tmpSet.get(gene)
                 unsure = np.sum([ 1 for m in mat if m[3] > 0 and m[5] in used2 ])
-                if unsure >= np.sum(mat.T[3]>0) * 0.8 :
+                if unsure >= np.sum(mat.T[3]>0) * 0.6 :
                     genes.pop(gene)
                 else :
                     cfl = conflicts.get(int(gene), {})
@@ -610,17 +610,17 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
                     if (not inparalog and np.min(mat[:, 3]) >= 10000 * params['clust_identity']) or len(mat) <= 1 :
                         new_groups[gene] = mat
                     else :
-                        to_run.append([mat, inparalog, len(clust_ref[ mat[0][0] ]), params['map_bsn']+'.seq.npz', global_file])
+                        to_run.append([mat, inparalog, len(clust_ref[ mat[0][0]*1000 ]), params['map_bsn']+'.seq.npz', global_file])
 
             working_groups = pool2.imap_unordered(filt_per_group, sorted(to_run, key=lambda r:(r[1], r[0].shape[0]*r[0].shape[0]*r[2]), reverse=True))
             #working_groups = [filt_per_group(d) for d in to_run]
             for working_group in working_groups :
-                gene = working_group[0][0][0]
+                gene = working_group[0][0][0]*1000
                 new_groups[gene] = working_group[0]
                 if len(working_group) > 1 :
-                    cfl = conflicts.get(int(gene), {})
+                    cfl = conflicts.get(gene, {})
                     for id, matches in enumerate(working_group[1:]) :
-                        ng = np.round(gene + 0.001*(id+2), 3)
+                        ng = gene + (id+2)
                         new_groups[ng] = matches
                         conflicts[ng] = { mid:cfl.pop(mid, {}) for mid in matches.T[5] }
                         scores[ng] = np.sum(np.abs(matches[np.unique(matches.T[1], return_index=True)[1]].T[2]))
@@ -641,7 +641,6 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
                         else :
                             used2.update( { int(k/10) for k in cfl.get(m[5],[]) + [m[5]*10] } )
                             
-                    genomes = np.unique(mat[~kept, 1])
                     mat = mat[kept]
                     new_groups[gene] = mat
         
@@ -656,7 +655,6 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
             if score < min_score :
                 break
             mat = new_groups.pop(gene)
-            x = encodes[int(gene)] + '/' + str(int(1000*(gene - int(gene)) + 0.5)) if isinstance(gene, float) else encodes[gene]
             # third, check its overlapping again
             paralog = False
             supergroup, used2 = {}, {}
@@ -666,7 +664,7 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
                 conflict = used.get(gid, None) if gid in used else used2.get(gid, None)
                 if conflict is not None :
                     if conflict < 0 :
-                        superC = pangenome[-(conflict+1)]
+                        superC = pangenome[-(conflict+1000)]
                         if superC not in supergroup :
                             supergroup[superC] = 0
                         if m[4] >= params['clust_identity'] * 10000 :
@@ -683,9 +681,9 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
                         g2, gs = int(gg/10), gg % 10
                         if gs == 1 :
                             if g2 not in used :
-                                used2[g2] = -(gene+1)
-                        else  :
-                            used2[g2] = gene+1 if gs == 2 else 0
+                                used2[g2] = -(gene+1000)
+                        else :
+                            used2[g2] = gene+1000 if gs == 2 else 0
                     
             if idens < (params['clust_identity']-0.02)*10000 :
                 scores.pop(gene)
@@ -719,10 +717,10 @@ def filt_genes(prefix, groups, ortho_groups, global_file, cfl_file, priorities,
             
             panList[pangene] = panList.get(pangene, set([])) | set(mat.T[1])
 
-            pangene_name = encodes[int(pangene)] + '/' + str(int(1000*(pangene - int(pangene)) + 0.5)) if isinstance(pangene, float) else encodes[pangene]
-            gene_name = encodes[int(gene)] + '/' + str(int(1000*(gene - int(gene)) + 0.5)) if isinstance(gene, float) else encodes[gene]
+            pangene_name = (encodes[int(pangene/1000)] + '/' + str(pangene % 1000)) if pangene % 1000 > 0 else encodes[int(pangene/1000)]
+            gene_name = (encodes[int(gene/1000)] + '/' + str(gene % 1000)) if gene % 1000 > 0 else encodes[int(gene/1000)]
             if len(pangenome) % 100 == 0 :
-                logger('{4} / {5}: pan gene "{3}" : "{0}" picked from rank {1} and score {2}'.format(encodes[mat[0][0]], min_rank, score/10000., pangene_name, len(pangenome), len(scores)+len(pangenome)))
+                logger('{4} / {5}: pan gene "{3}" : "{0}" picked from rank {1} and score {2}'.format(gene_name, min_rank, score/10000., pangene_name, len(pangenome), len(scores)+len(pangenome)))
             mat_out.append([pangene_name, gene_name, min_rank, mat])
     mat_out.append([0, 0, 0, []])
     return 
@@ -1491,9 +1489,10 @@ def write_output(prefix, prediction, genomes, clust_ref, encodes, old_prediction
         glen = np.mean([len(s) for s in alleles[gene]]) if len(alleles[gene]) else 0
         if glen < params['min_cds'] :
             removed[gene] = 1
-        elif (stat[1]) <= (stat[0])*untrusted[1] and \
-                (stat[3]) <= (stat[2])*untrusted[1] and \
-                glen < untrusted[0] :
+        elif (stat[1]+0.5)*(stat[3]+0.5) < (stat[0]+1)*untrusted[1]*(stat[2]+1)*untrusted[1] and \
+                glen <= untrusted[0] :
+            removed[gene] = 1
+        elif stat[3] == 0 and len(re.findall(r'/[\.\d]+$', gene)) :
             removed[gene] = 1
     for pid2, pid1 in sorted(toMerge.items()) :
         p1, p2 = prediction[pid1], prediction[pid2]
@@ -1678,14 +1677,14 @@ PEPPA.py
     parser.add_argument('--link_diff', help='Form a linked block when the covered regions in the reference gene \nand the queried genome differed by no more than this value. Default: 1.2', default=1.2, type=float)
 
     parser.add_argument('--allowed_sigma', help='Allowed number of sigma for paralogous splitting. \nThe larger, the more variations are kept as inparalogs. Default: 3.', default=3., type=float)
-    parser.add_argument('--pseudogene', help='A match is reported as pseudogene if its coding region is less than this amount of the reference gene. Default: 0.7', default=.7, type=float)
-    parser.add_argument('--untrusted', help='FORMAT: l,p; A gene is not reported if it is shorter than l and present in less than p of prior annotations. Default: 300,0.35', default='300,0.35')
+    parser.add_argument('--pseudogene', help='A match is reported as a pseudogene if its coding region is less than a proportion of the reference gene. Default: 0.7', default=.7, type=float)
+    parser.add_argument('--untrusted', help='FORMAT: l,p; A gene is not reported if it is not greater than "l" and present in less than "p" of GFF files. Default: 450,0.35', default='450,0.35')
     parser.add_argument('--continue', help='continue from a previously stopped run.', default=False, action='store_true')
 
-    parser.add_argument('--intron', help="Enable this to allow multiple CDSs being concatenated if they are under the same name. This is still under development. ", default=False, action='store_true')
+    #parser.add_argument('--intron', help="Enable this to allow multiple CDSs being concatenated if they are under the same name. This is still under development. ", default=False, action='store_true')
     parser.add_argument('--feature', help='feature to extract. Be cautious to change this value. DEFAULT: CDS', default='CDS')
-    parser.add_argument('--noncoding', help='Set to noncoding mode. This is still under development. Equals to \n"--nucl --incompleteCDS sife --untrusted 0,1"', default=False, action='store_true')
-    parser.add_argument('--metagenome', help='Set to metagenome mode. This is still under development. Equals to \n"--nucl --incompleteCDS sife --clust_identity 0.99 --clust_match_prop 0.8 --match_identity 0.98 --orthology sbh --untrusted 0,1"', default=False, action='store_true')
+    parser.add_argument('--noncoding', help='Set to noncoding mode. This is still under development. Equals to \n"--nucl --incompleteCDS sife"', default=False, action='store_true')
+    parser.add_argument('--metagenome', help='Set to metagenome mode. This is still under development. Equals to \n"--nucl --incompleteCDS sife --clust_identity 0.99 --clust_match_prop 0.8 --match_identity 0.98 --orthology sbh"', default=False, action='store_true')
     parser.add_argument('--testunit', help='download four E. coli ST131 genomes for testing of PEPPA.', default=False, action='store_true')
 
     params = parser.parse_args(a)
@@ -1701,7 +1700,6 @@ PEPPA.py
     if params.noncoding :
         params.noDiamond = True
         params.incompleteCDS = 'sife'
-        params.untrusted = '0,1'
     elif params.metagenome :
         params.noDiamond = True
         params.incompleteCDS = 'sife'
@@ -1709,7 +1707,6 @@ PEPPA.py
         params.clust_match_prop = 0.8
         params.match_identity = 0.98
         params.orthology = 'sbh'
-        params.untrusted = '0,1'
     params.untrusted = [float(p) for p in params.untrusted.split(',')]
     params.incompleteCDS = params.incompleteCDS.lower()
     if params.noNeighborCheck :
@@ -1882,7 +1879,7 @@ def ortho() :
         writeProcess.start()
         gene_scores = initializing(params['map_bsn'], params.get('global', None))
         with MapBsn(params['map_bsn']+'.tab.npz') as tab_conn :
-            filt_genes(params['prefix'], tab_conn, np.load(params['self_bsn'], allow_pickle=True), params['global'], params['map_bsn']+'.conflicts.npz', priorities, gene_scores, encodes)
+            filt_genes(tab_conn, np.load(params['self_bsn'], allow_pickle=True), params['global'], params['map_bsn']+'.conflicts.npz', priorities, gene_scores, encodes)
         writeProcess.join()
         shutil.move(params['prediction']+'.working', params['prediction'])
     else :
